/* Phase Cycled TROSY Sequence: designed to select the narrow component of the N-H doublet in 15N and 1H
   dimensions.
   
  Time shared, by the parameter ilve = 1,2
  
  set array = 'phase2,phase,pcyc,ilve'
  set all those parameters above to 1,2
   
   Modified to ensure zero 1st order phase correction in the 15N dimension */
   
   /* Ananya Majumdar, Jan 10, 2008 */

/* SHM modified July 17 2013 */



/* This choses the HNalpha */

#include <standard.h>
#include <math.h>
#define Pi M_PI

static double d2_init = 0.0,
              d3_init = 0.0; 


static int  

 phi1 [2] = {3,1}, /* 13C 90 pulse before t1 y,-y phase Cycled y,x, y,x  */
 phi2 [2] = {1,3}, /* 15N 90 pulse before t1 y,-y phase Cycled same for e1 to e4 */
 phi3 [1] = {3},   /* 13C 90 pulse after 13C t1 y,x,x,y */
 phi4 [1] = {3},   /* 15N 1st and 2nd 90 pulse of the ST2PT -y,-x,-y,-x */
   rec[2] = {3,1}, /* receiver y - y follows 13C & 15 N phase cycling */
 phi5 [1] = {0},  /* 1H 2nd 90 of ST2PT x,y,x,y */
 phi6 [1] = {2},  /* 1H last 90 pulse of ST2PT b4 t2 -x, -y, x,y */
 phi7 [1] = {0},  /* 15N last 90 pulse of ST2PT b4 t2 x,y,x,y */
 phi8 [1] = {3},  /* 1H 180 in t2 depends on t9, since 1st is E1x, this is y */
 phi9 [1] = {0},  /* 1H 90 before mixing time */
 phi10 [1] = {3}, /* 1H 1st 90 of ST2PT this is same as phi 1, y,x,y,x, but is not phase cycled */
 phi12 [1] = {0}, /* H2O 1st ST2PT 1st pulse  E1 to E4x = x,x,x,x ; E1 to E4y = y,-x, y, x  */
 phi13 [1] = {0}, /* H2O 1st ST2PT 2nd pulse  w2  E1 to E4x = x,x,x,x ; E1 to E4y = y,y,y,y */
 phi14 [1] = {0}, /* H2O 2nd ST2PT 1st pulse  w3  E1 to E4x = x,y,x,y ; E1 to E4y = y,y,y,y */
 phi15 [1] = {1}; /* H2O 2nd ST2PT 2nd pulse  w4  E1 to E4x = -y,-y,y,y ; E1 to E4y = -y,-y,-y,-y */
 



pulsesequence()
{
/* DECLARE VARIABLES */

 char 	  f1180[MAXSTR], f2180[MAXSTR];
            
	     
 int	     ilve =getval("ilve"),		/* interleave with values of 1,2*/ 
 	     pcyc = getval("pcyc"),		/* use values of 1,2 */   
             phase = getval("phase"),		/* use phase = 1,2 */
	     phase2 = getval("phase2");		/* use phase2 = 1,2 */
	     
	     		
 double      
	     tnh = getval("tnh")*0.001,		/* 1/4JNH for H-N inept steps (ms) */
	     tnh2 = getval("tnh2")*0.001,	/* 1/4JNH for TROSY sequence (ms) */
	    
	     delC = getval("delC")*0.001,  	 /* 1/4JCH for HSQC (ms), ~ 2 ms for 125 Hz */
	     kappa, 				/* adjustment for 15N evolution after 13C evolution in TS */
	     
	   			
	     
             pwx2 = getval("pwx2"),		/* hard 15N PW90 */
	     dhpwr2 = getval("dhpwr2"),		/* power level for 15N hard pulses */
	     
	     swC = getval("swC"),		/* spectral width along f1 */
	     sw2 = getval("sw2"),      		/* spectral width along f2 */
	     swN = getval("swN"),		/* spectral width for 15N */
	     
	     pwx = getval("pwx"),		/* hard 13C pw90 */
	     pwxlvl = getval("pwxlvl"), 	/* power level for 13C hard pulses */
	     
	    
	     wgpwr = getval("wgpwr"),		/* watergate power level */
	     pwg = getval("pwg"),		/* pw90 @ wgpwr */
	     phwg = getval("phwg"),		/* fine phase adjustment for watergate pulse */
	                  				   	         	     
	     gzlvl0 = getval("gzlvl0"),		/* clean up gradient at the beginning of the sequence */
	     gt0 = getval("gt0"),
	     
	     gzlvl1 = getval("gzlvl1"),   	/* cleanup gradient around 180 pulses in first INEPT (small, almost zero) */      
             gt1 = getval("gt1"), 
	                       
             gzlvl2 = getval("gzlvl2"),		/* cleanup gradient in 1st segment of TROSY readout ~ 4000 */
	     gt2 = getval("gt2"),		/* ~ 250e-6 */
	     
	     gzlvl3 = getval("gzlvl3"),		/* watergate gradient in 2nd segment of TROSY readout ~ 15000 */
	     gt3 = getval("gt3"),		/* 250 - 300e-6 */
	    
	    
	     gzlvl4 = getval("gzlvl4"),		/* watergate gradient before acquisition ~ 15000 */
	     gt4 = getval("gt4"), 
	     
	     gzlvl5 = getval("gzlvl5"),		
	     gt5 = getval("gt5"),
	    
	    
	    gzlvl6 = getval("gzlvl6"),		
	     gt6 = getval("gt6"),
	    
	    
	     gstab = getval("gstab")*1.0e-6,	/* gradient stabilization delay 250 - 350 us */
	
	     mix = getval("mix")*0.001;     /* NOESY mixing time in ms */
	     
	 
	
	

 double     tau1,	      	    		/* t1/2 */
	    tau2;				/* t2/2 */
			 
 
    double	     simpulse_comp; /* delay to compensate for 1st order phase correction in t1, 13C */
             			
 	     
	     
 


/* make sure all decoupling is OFF */

    if(dm[A] == 'y')
    {printf("Decoupling on Dec1 during t1! ");abort(1);}
    
  /*  if(dm[B] == 'y')
    {printf("Decoupling on Dec1 during acquisition!  ");abort(1);} */
    
    if(dm2[A] == 'y')
    {printf("Decoupling on Dec2 during t1!  ");abort(1);}
    
/*   if(dm2[B] == 'y')
    {printf("Decoupling on Dec2 during acquisition!"); abort(1);}  */

    

/* set phase tables */

	settable (t1, 2, phi1);
	settable (t2, 2, phi2);
	settable (t3, 1, phi3);
	settable (t4, 1, phi4);
	settable (t5, 1, phi5);
    settable (t6, 1, phi6);
    settable (t7, 1, phi7);
 	settable (t8, 1, phi8);
	settable (t9, 1, phi9);
    settable (t10,1, phi10);
	settable (t12, 1, phi12);
	settable (t13, 1, phi13);
    settable (t14, 1, phi14);
    settable (t15, 1, phi15);
    settable(t31, 2, rec);
	
      
 /* watergate */
 
   if(phwg < 0.0) phwg += 360.0;
   initval(phwg,v8);
   
   
     /*   INITIALIZE VARIABLES   */
   
   
   /* Adjustment of 15N evolution period */

	kappa = (2*swC/swN - 1);	


/* balancing delays  */ 
  
   if (pwx2 > 2 * pw) { simpulse_comp = pwx2; }    /* tau 1 for 13C compensation */
  else { simpulse_comp = 2 * pw; }
      
 
	
/* Phase incrementation for States quadrature detection along t1 */

/* use original phase table for E1 , pcyc = 1, phase = 1 phase = 1  */

/* phase = 1,2 */

   if  ((pcyc == 1) && (phase == 1) && (phase2 == 1) && (ilve == 1))  /* E1x, 15N + 13C */
  { 
    tsadd(t31,0, 4);
   }
 
  if  ((pcyc == 1) && (phase == 1) && (phase2 == 1) && (ilve == 2))  /* E1x, 15N - 13C */
  { 
    tsadd(t31,0, 4);
    tsadd(t3,2,4); /* 13C reversed */
   }
    

/**********/



  if  ((pcyc == 2) && (phase == 1) && (phase2 == 1) && (ilve == 1))  /* E2 y, 15N + 13C  */
  {   tsadd(t1,1,4);
      tsadd(t3,1,4);
      tsadd(t4,3,4);
      tsadd(t5,3,4);
      tsadd(t6,3,4);
      tsadd(t7,1,4);
      tsadd(t8,1,4);
      tsadd(t9,3,4); /* y NOESY*/
      tsadd(t10,1,4);
      tsadd(t12,2,4);
      tsadd(t13,3,4);
      tsadd(t14,3,4);
      
      
   }
  
  if  ((pcyc == 2) && (phase == 1) && (phase2 == 1) && (ilve == 2))  /* E2 y, 15N - 13C  */
  {   tsadd(t1,1,4);
      tsadd(t3,3,4);  /* 13C reversed */
      tsadd(t4,3,4);
      tsadd(t5,3,4);
      tsadd(t6,3,4);
      tsadd(t7,1,4);
      tsadd(t8,1,4);
      tsadd(t9,3,4); /* y NOESY*/
      tsadd(t10,1,4);
      tsadd(t12,2,4);
      tsadd(t13,3,4);
      tsadd(t14,3,4);
      
   }
 
 
 
 /**********/
  
  if  ((pcyc == 1) && (phase == 2) && (phase2 == 1) && (ilve == 1))  /* E3 x, 15N + 13C   */
  {  tsadd(t3,1,4);
     tsadd(t6,2,4);
     tsadd(t15,2,4);
       
  }


if  ((pcyc == 1) && (phase == 2) && (phase2 == 1) && (ilve == 2))  /* E3 x, 15N - 13C   */
{   tsadd(t3,3,4); /* 13C reversed */
    tsadd(t6,2,4);
    tsadd(t15,2,4);
  }

/**********/


    
  if  ((pcyc == 2) && (phase == 2) && (phase2 == 1) && (ilve == 1))  /* E4 y, 15N + 13C  */
  {  tsadd(t1,1,4);
     tsadd(t4,3,4);
     tsadd(t5,3,4);
     tsadd(t6,1,4);
     tsadd(t7,1,4);
     tsadd(t8,1,4);
     tsadd(t9,3,4); /* y NOESY*/
     tsadd(t10,1,4);
     tsadd(t13,3,4);
     tsadd(t14,3,4);
  }

if  ((pcyc == 2) && (phase == 2) && (phase2 == 1) && (ilve == 2))  /* E4 y, 15N - 13C  */
{   tsadd(t1,1,4);
    tsadd(t4,3,4);
    tsadd(t5,3,4);
    tsadd(t6,1,4);
    tsadd(t7,1,4);
    tsadd(t8,1,4);
    tsadd(t9,3,4); /* y NOESY*/
    tsadd(t10,1,4);
    tsadd(t13,3,4);
    tsadd(t14,3,4);
    tsadd(t3,2,4); /* 13C reversed */
    
  }

/**********/

/* phase2 = 2 */

   if  ((pcyc == 1) && (phase == 1) && (phase2 == 2) && (ilve == 1))  /* E1 y, 15N + 13C  */ 
  {   
      tsadd(t8,1,4);
      tsadd(t9,3,4); /* y NOESY*/
      tsadd(t12,3,4);
      tsadd(t13,3,4);
      tsadd(t14,3,4);
   }
   
   if  ((pcyc == 1) && (phase == 1) && (phase2 == 2) && (ilve == 2))  /* E1 y, 15N - 13C  */ 
   {   
   	tsadd(t8,1,4);
       tsadd(t9,3,4); /* y NOESY*/
       tsadd(t12,3,4);
       tsadd(t13,3,4);
       tsadd(t14,3,4);
       tsadd(t3,2,4);
   }
   
   
/**********/      
   
   if  ((pcyc == 2) && (phase == 1) && (phase2 == 2) && (ilve == 1))  /* E2x, 15N + 13C */
  {  tsadd(t1,1,4);
     tsadd(t3,1,4);
     tsadd(t4,3,4);
     tsadd(t5,3,4);
     tsadd(t6,3,4);
     tsadd(t7,1,4);
     tsadd(t10,1,4);
     tsadd(t14,3,4);
   }
   
  if  ((pcyc == 2) && (phase == 1) && (phase2 == 2) && (ilve == 2))  /* E2x, 15N - 13C*/
  {   tsadd(t1,1,4);
      tsadd(t3,3,4); /* 13C reversed */
      tsadd(t4,3,4);
      tsadd(t5,3,4);
      tsadd(t6,3,4);
      tsadd(t7,1,4);
      tsadd(t10,1,4);
      tsadd(t14,3,4);
   }  
   
   
/**********/  
 
   if  ((pcyc == 1) && (phase == 2) && (phase2 == 2) && (ilve == 1))  /* E3 y, 15N + 13C */
  {  
  	tsadd(t3,1,4);
     tsadd(t6,2,4);
     tsadd(t8,1,4);
     tsadd(t9,3,4); /* y NOESY */
     tsadd(t12,3,4);
     tsadd(t13,3,4);
     tsadd(t14,3,4);
      
     
   }


  if  ((pcyc == 1) && (phase == 2) && (phase2 == 2) && (ilve == 2))  /* E3 y, 15N - 13C */
  {   
  	tsadd(t3,3,4); /* 13C reverse */
      tsadd(t6,2,4);
      tsadd(t8,1,4);
      tsadd(t9,3,4); /* y NOESY */
      tsadd(t12,3,4);
      tsadd(t13,3,4);
      tsadd(t14,3,4);
     
   }


/**********/

   
   if  ((pcyc == 2) && (phase == 2) && (phase2 == 2) && (ilve == 1))  /* E4 x, 15N + 13C  */
   { tsadd(t1,1,4);
     tsadd(t4,3,4);
     tsadd(t5,3,4);
     tsadd(t6,1,4);
     tsadd(t7,1,4);
     tsadd(t10,1,4);
     tsadd(t14,3,4);
     tsadd(t15,2,4);
   }
   	
	
 if  ((pcyc == 2) && (phase == 2) && (phase2 == 2) && (ilve == 2))  /* E4 x, 15N - 13C  */
 {   tsadd(t1,1,4);
     tsadd(t4,3,4);
     tsadd(t5,3,4);
     tsadd(t6,1,4);
     tsadd(t7,1,4);
     tsadd(t10,1,4);
     tsadd(t14,3,4);
     tsadd(t15,2,4);
     tsadd(t3,2,4); /* 13C reverse */
   }	
	


	/* States-TPPI along f1 */
 
 	
 if (d2_index % 2)  	
	{
        tsadd(t1,2,4);
        tsadd(t2,2,4);
        tsadd(t31,2,4);
      }
      
  

	/* No States-TPPI along f2, axial peaks are fine since 1H is aliased */
	
	


/* set initial t1 delay and adjust for lp1=-180 rp1=90 if f1180='y' */
 

 
  tau1 = (d2_index/swC);
  
 /*  if(f1180[A] == 'y')  tau1 += ( 1.0/(2.0*sw1) );   */
  
   tau1 /= 2.0; 
	




/* set initial t2 delay */ 
	
	tau2 = (d3_index/sw2);
/*	if(f2180[A] == 'y')  tau2 += ( 1.0/(2.0*sw2) );  */
 
   	tau2 /= 2.0; 
	
	
			
	
	 
/* BEGIN ACTUAL PULSE SEQUENCE */

status(A);

/* relaxation delay */

    rcvroff();  
     
   	obspower(tpwr);               
   	decpower(pwxlvl); decphase(zero);
	dec2power(dhpwr2); dec2phase(zero); 
              
   	delay(d1);
	
   
  /* pulse sequence begins */
  
   
  decrgpulse(pwx, zero, 2.0e-6,2.0e-6);  /* destroy native 13C magnetization */
  zgradpulse(gzlvl0,gt0);
  delay(gstab);				
  
    
  rgpulse(pw,zero,2.0e-6,2.0e-6); /* Hy */
    
  	delay(2.0e-6);
  	zgradpulse(gzlvl1,gt1);
	
	delay(delC - gt1 - 4.0e-6);
	decrgpulse(2.0*pwx, zero, 0.0, 0.0);
	
	
  	decphase(t1); txphase(three);
  	delay(tnh - delC - 2 * pwx);
	
  sim3pulse(2.0*pw,0.0,2.0*pwx2,three,zero,zero,2.0e-6,2.0e-6);		
  
  	delay(2.0e-6);
  	zgradpulse(gzlvl1,gt1);
  
  	txphase(one); dec2phase(t2);
  	delay(tnh - 4.0e-6 - gt1);   /* HxNz */

  rgpulse(pw,one,2.0e-6,2.0e-6);     /* HzNz */ 
  
  	obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(zero); 
  shaped_pulse("hard",pwg,zero,3.0e-6,3.0e-6);  
        obspower(tpwr); xmtrphase(zero); txphase(zero);
	
	delay(10e-6);
        
	zgradpulse(gzlvl6,gt6);   /* purge all transverse components */
	delay(gstab);
 	
	
	/* TS period */
	
	decrgpulse(pwx, t1, 0.0, 2.0e-6);
	
	delay (d2_init + simpulse_comp + 4.0e-6);
	
	decrgpulse(2.0*pwx, t1, 0.0, 2.0e-6);  
	
	delay(tau1);
 
        sim3pulse(2.0*pw,0.0,pwx2,zero,zero,t2,0.0,2.0e-6);
  
  	delay(tau1);
	
	decrgpulse(pwx, t3, 2.0e-6, 0.0); 	/* 13C evolution period ends */
		
	delay(kappa*tau1);		/* 15N evolution period ends */
	
	dec2rgpulse(2.0*pwx2,zero,0.0,2.0e-6);
	
	delay (pwx + d2_init);
		
	txphase(t10);
	
/* Back transfer for 13C and 15N pathways */ 	  
	
  sim3pulse(pw,0.0,pwx2,t10,zero,t4,2.0e-6,0.0);
    
    obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(t12);
    shaped_pulse("hard",pwg,t12,3.0e-6,3.0e-6);
	
	zgradpulse(gzlvl2,gt2);
    obspower(tpwr); xmtrphase(zero); txphase(zero); 
	delay(gstab);
	  
   	delay(delC - gt2 - gstab - pwx - pwg - 6.0e-6);
	
	decrgpulse(2.0*pwx, zero, 0.0, 0.0);
  
  	delay(tnh2 - delC - pwx);   
  
  					
   sim3pulse(2.0*pw,0.0,2.0*pwx2,zero,zero,zero,0.0,0.0);
 	  
  	delay(tnh2 - gt2 - gstab - pwg - 6.0e-6);
	
    obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(t13);
	
    zgradpulse(gzlvl2,gt2);
	delay(gstab);
    
    shaped_pulse("hard",pwg,t13,3.0e-6,3.0e-6);
    obspower(tpwr); xmtrphase(zero); txphase(t5);
    
 sim3pulse(pw,0.0,pwx2,t5,zero,t4,0.0,2.0e-6);    /* first part of ST2PT complete */
	
  	obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(t14);
   
    shaped_pulse("hard",pwg,t14,3.0e-6,3.0e-6);
	
	zgradpulse(gzlvl3,gt3);
    obspower(tpwr); xmtrphase(zero); txphase(zero); 
	delay(gstab);
  	
  	delay(tnh2 - gt3 - gstab - pwg - 6.0e-6);
	
	sim3pulse(2.0*pw,0.0,2.0*pwx2,zero,zero,zero,0.0,0.0);
	
	delay(tnh2 - gt3 - gstab - pwg - 6.0e-6);
    
    obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(t15);
    
	zgradpulse(gzlvl3,gt3);
    delay(gstab);
    
    shaped_pulse("hard",pwg,t15,3.0e-6,3.0e-6);
    obspower(tpwr); xmtrphase(zero); txphase(t6);
	
	sim3pulse(pw,0.0,pwx2,t6,zero,t7,2.0e-6,2.0e-6);
	
	/* Second part of ST2PT complete */
	
	/* t2 encoding before Noesy */
	
	delay(tau2);  
	
	delay(3.0e-6);
	
	decrgpulse(2.0*pwx,zero,2.0e-6,2.0e-6); 
	
	delay(3.0e-6);
	
	delay(tau2);  
	
	delay(3.0e-6);
	
	rgpulse(2.0*pw,t8,2.0e-6,2.0e-6);
	
    txphase(t9);
	
    delay(0.5*(pwx2 - pw) + 17.0e-6 + 2.0*pwx + d3_init);
	
  
	rgpulse(pw,t9,2.0e-6,0.0);
	
	delay(mix - pwg - 8.0e-6 - gt5 -gstab);
	
	zgradpulse(gzlvl5,gt5);
	delay(gstab);
	
	
	/* flip back */
	
	obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(two); 
	shaped_pulse("hard",pwg,two,3.0e-6,3.0e-6);
	obspower(tpwr); xmtrphase(zero); txphase(zero);
	
	
	rgpulse(pw, zero,2.0e-6,2.0e-6);
			
	/* watergate*/
	
	delay(2.0e-6);   
	zgradpulse(gzlvl4,gt4); /* water is kept along Z */
	delay(gstab);
	
	
	obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(zero);
	
	shaped_pulse("hard",pwg,zero,3.0e-6,3.0e-6);
	obspower(tpwr); xmtrphase(zero); txphase(three);
	
	rgpulse(2.0*pw,three,2.0e-6,2.0e-6);
	
  	obspower(wgpwr); obsstepsize(1.0); xmtrphase(v8); txphase(two);
	
	shaped_pulse("hard",pwg,two,3.0e-6,3.0e-6);
	obspower(tpwr); xmtrphase(zero); txphase(zero); 
	
	delay(2.0e-6);
	zgradpulse(gzlvl4,gt4);
	delay(gstab);
	
	delay(5.0e-6);
	
	dec2power(dpwr2);
	
	decpower(dpwr);
	

/* begin acquisition */
status(B);
     setreceiver(t31);     
}
